@@ -13,6 +13,7 @@ from dataclasses_json import Undefined, config, DataClassJsonMixin
 
 from aiohttp_sse_client.client import MessageEvent
 
+from .const import Events
 from .common import HomeConnectError
 from .callback_registery import CallbackRegistry
 from .appliance import Appliance
@@ -66,7 +67,13 @@ class HomeConnect(DataClassJsonMixin):
     _callbacks:Optional[CallbackRegistry] = field(default_factory=lambda: CallbackRegistry(), metadata=config(encoder=lambda val: None, exclude=lambda val: True))
 
     @classmethod
-    async def async_create(cls, am:AuthManager, json_data:str=None, delayed_load:bool=False, refresh:RefreshMode=RefreshMode.DYNAMIC_ONLY, auto_update:bool=False) -> HomeConnect:
+    async def async_create(cls,
+        am:AuthManager,
+        json_data:str=None,
+        delayed_load:bool=False,
+        refresh:RefreshMode=RefreshMode.DYNAMIC_ONLY,
+        auto_update:bool=False,
+        lang:str=None) -> HomeConnect:
         """ Factory for creating a HomeConnect object - DO NOT USE THE DEFAULT CONSTRUCTOR
 
         Parameters:
@@ -80,7 +87,7 @@ class HomeConnect(DataClassJsonMixin):
 
         If auto_update is set to False then subscribe_for_updates() should be called to receive real-time updates to the data
         """
-        api = HomeConnectApi(am)
+        api = HomeConnectApi(am, lang)
         hc:HomeConnect = None
         if json_data:
             try:
@@ -134,7 +141,7 @@ class HomeConnect(DataClassJsonMixin):
         try:
             if refresh == self.RefreshMode.NOTHING:
                 for appliance in self.appliances.values():
-                    await self._callbacks.async_broadcast_event(appliance, "PAIRED")
+                    await self._callbacks.async_broadcast_event(appliance, Events.PAIRED)
 
             else:
                 response = await self._api.async_get('/api/homeappliances')
@@ -153,13 +160,13 @@ class HomeConnect(DataClassJsonMixin):
                             elif ha['haId'] not in self.appliances or refresh==self.RefreshMode.ALL:
                                 appliance = await Appliance.async_create(self, ha)
                                 self.appliances[ha['haId']] = appliance
-                            await self._callbacks.async_broadcast_event(self.appliances[ha['haId']], "PAIRED")
+                            await self._callbacks.async_broadcast_event(self.appliances[ha['haId']], Events.PAIRED)
                             _LOGGER.debug("Loadded appliance: %s", self.appliances[ha['haId']].name)
 
                 # clear appliances that are no longer paired with the service
                 for haId in self.appliances.keys():
                     if haId not in haid_list:
-                        await self._callbacks.async_broadcast_event(self.appliances[haId], "DEPAIRED")
+                        await self._callbacks.async_broadcast_event(self.appliances[haId], Events.DEPAIRED)
                         del self.appliances[haId]
 
             self.status |= self.HomeConnectStatus.LOADED
@@ -286,22 +293,22 @@ class HomeConnect(DataClassJsonMixin):
             self._last_update = datetime.now()
         elif event.type == 'PAIRED':
             self.appliances[haid] = await Appliance.async_create(self._api, haId=haid)
-            await self._callbacks.async_broadcast_event(self.appliances[haid], event.type)
+            await self._callbacks.async_broadcast_event(self.appliances[haid],  Events.PAIRED)
         elif event.type == 'DEPAIRED':
             if haid in self.appliances:
-                await self._callbacks.async_broadcast_event(self.appliances[haid], event.type)
+                await self._callbacks.async_broadcast_event(self.appliances[haid], Events.DEPAIRED)
                 del self.appliances[haid]
-        elif event.type == 'DISCONNECTED':
+        elif event.type =='DISCONNECTED':
             if haid in self.appliances:
                 await self.appliances[haid].async_set_connection_state(False)
-                await self._callbacks.async_broadcast_event(self.appliances[haid], event.type)
+                await self._callbacks.async_broadcast_event(self.appliances[haid], Events.DISCONNECTED)
         elif event.type == 'CONNECTED':
             if haid in self.appliances:
                 await self.appliances[haid].async_set_connection_state(True)
-                await self._callbacks.async_broadcast_event(self.appliances[haid], event.type)
+                await self._callbacks.async_broadcast_event(self.appliances[haid], Events.CONNECTED)
             else:
                 self.appliances[haid] = await Appliance.async_create(self._api, haId=haid)
-                await self._callbacks.async_broadcast_event(self.appliances[haid], "PAIRED")
+                await self._callbacks.async_broadcast_event(self.appliances[haid], Events.PAIRED)
         else:
             # Type is NOTIFY or EVENT
             data = json.loads(event.data)
@@ -309,10 +316,18 @@ class HomeConnect(DataClassJsonMixin):
                 for item in data['items']:
                     haid = self._get_haId_from_event(item) if 'uri' in item else haid
                     if haid in self.appliances:
-                        if item['key'] in ['BSH.Common.Root.SelectedProgram', 'BSH.Common.Status.OperationState']:
-                            await self[haid].async_fetch_data(include_static_data=False)
-                        else:
-                            await self._callbacks.async_broadcast_event(self.appliances[haid], item['key'], item['value'])
+                        appliance = self.appliances[haid]
+                        await appliance.async_update_data(item['key'], item['value'])
+
+                        # if item['key'] in ['BSH.Common.Root.SelectedProgram', 'BSH.Common.Status.OperationState']:
+                        #     await self[haid].async_fetch_data(include_static_data=False)
+                        #     if item['value'] == 'BSH.Common.EnumType.OperationState.Run':
+                        #         await self._callbacks.async_broadcast_event(appliance, Events.PROGRAM_STARTED)
+                        #     elif item['value'] == 'BSH.Common.EnumType.OperationState.Finished':
+                        #         await self._callbacks.async_broadcast_event(appliance, Events.PROGRAM_FINISHED)
+
+                        # else:
+                        #     await self._callbacks.async_broadcast_event(appliance, item['key'], item['value'])
 
 
     def _get_haId_from_event(self, event:dict):
